"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertAttributes = void 0;
const common_1 = require("./common");
const mustache_1 = require("./mustache");
const text_1 = require("./text");
const errors_1 = require("../../errors");
/** Convert for Attributes */
function* convertAttributes(attributes, parent, ctx) {
    for (const attr of attributes) {
        if (attr.type === "Attribute") {
            yield convertAttribute(attr, parent, ctx);
            continue;
        }
        if (attr.type === "Spread") {
            yield convertSpreadAttribute(attr, parent, ctx);
            continue;
        }
        if (attr.type === "Binding") {
            yield convertBindingDirective(attr, parent, ctx);
            continue;
        }
        if (attr.type === "EventHandler") {
            yield convertEventHandlerDirective(attr, parent, ctx);
            continue;
        }
        if (attr.type === "Class") {
            yield convertClassDirective(attr, parent, ctx);
            continue;
        }
        if (attr.type === "Transition") {
            yield convertTransitionDirective(attr, parent, ctx);
            continue;
        }
        if (attr.type === "Animation") {
            yield convertAnimationDirective(attr, parent, ctx);
            continue;
        }
        if (attr.type === "Action") {
            yield convertActionDirective(attr, parent, ctx);
            continue;
        }
        if (attr.type === "Let") {
            yield convertLetDirective(attr, parent, ctx);
            continue;
        }
        if (attr.type === "Ref") {
            throw new errors_1.ParseError("Ref are not supported.", attr.start, ctx);
        }
        throw new errors_1.ParseError(`Unknown directive or attribute (${attr.type}) are not supported.`, attr.start, ctx);
    }
}
exports.convertAttributes = convertAttributes;
/** Convert for Attribute */
function convertAttribute(node, parent, ctx) {
    const attribute = Object.assign({ type: "SvelteAttribute", boolean: false, key: null, value: [], parent }, ctx.getConvertLocation(node));
    const keyStart = ctx.code.indexOf(node.name, node.start);
    const keyRange = { start: keyStart, end: keyStart + node.name.length };
    attribute.key = Object.assign({ type: "SvelteName", name: node.name, parent: attribute }, ctx.getConvertLocation(keyRange));
    if (node.value === true) {
        // Boolean attribute
        attribute.boolean = true;
        ctx.addToken("HTMLIdentifier", keyRange);
        return attribute;
    }
    for (let index = 0; index < node.value.length; index++) {
        const v = node.value[index];
        if (v.type === "AttributeShorthand") {
            const key = Object.assign(Object.assign({}, attribute.key), { type: "Identifier" });
            const sAttr = {
                type: "SvelteShorthandAttribute",
                key,
                value: key,
                parent,
                loc: attribute.loc,
                range: attribute.range,
            };
            ctx.scriptLet.addExpression(key, sAttr, null, (es) => {
                sAttr.key = es;
                sAttr.value = es;
            });
            return sAttr;
        }
        if (v.type === "Text") {
            const next = node.value[index + 1];
            if (next && next.start < v.end) {
                // Maybe bug in Svelte can cause the completion index to shift.
                // console.log(ctx.getText(v), v.data)
                v.end = next.start;
            }
            attribute.value.push(text_1.convertTextToLiteral(v, attribute, ctx));
            continue;
        }
        if (v.type === "MustacheTag") {
            const mustache = mustache_1.convertMustacheTag(v, attribute, ctx);
            attribute.value.push(mustache);
            continue;
        }
        const u = v;
        throw new errors_1.ParseError(`Unknown attribute value (${u.type}) are not supported.`, u.start, ctx);
    }
    // Not required for shorthands. Therefore, register the token here.
    ctx.addToken("HTMLIdentifier", keyRange);
    return attribute;
}
/** Convert for Spread */
function convertSpreadAttribute(node, parent, ctx) {
    const attribute = Object.assign({ type: "SvelteSpreadAttribute", argument: null, parent }, ctx.getConvertLocation(node));
    const spreadStart = ctx.code.indexOf("...", node.start);
    ctx.addToken("Punctuator", {
        start: spreadStart,
        end: spreadStart + 3,
    });
    ctx.scriptLet.addExpression(node.expression, attribute, null, (es) => {
        attribute.argument = es;
    });
    return attribute;
}
/** Convert for Binding Directive */
function convertBindingDirective(node, parent, ctx) {
    const directive = Object.assign({ type: "SvelteDirective", kind: "Binding", name: null, modifiers: node.modifiers, expression: null, parent }, ctx.getConvertLocation(node));
    processDirective(node, directive, ctx, (expression) => {
        return ctx.scriptLet.addExpression(expression, directive, null, (es, { getInnermostScope }) => {
            directive.expression = es;
            const scope = getInnermostScope(es);
            const reference = scope.references.find((ref) => ref.identifier === es);
            if (reference) {
                // The bind directive does read and write.
                reference.isWrite = () => true;
                reference.isWriteOnly = () => false;
                reference.isReadWrite = () => true;
                reference.isReadOnly = () => false;
                reference.isRead = () => true;
            }
        });
    });
    return directive;
}
/** Convert for EventHandler Directive */
function convertEventHandlerDirective(node, parent, ctx) {
    const directive = Object.assign({ type: "SvelteDirective", kind: "EventHandler", name: null, modifiers: node.modifiers, expression: null, parent }, ctx.getConvertLocation(node));
    const isCustomEvent = parent.parent.type === "SvelteElement" &&
        (parent.parent.kind === "component" || parent.parent.kind === "special");
    processDirective(node, directive, ctx, buildProcessExpressionForExpression(directive, ctx, isCustomEvent
        ? "(e:CustomEvent<any>)=>void"
        : `(e:HTMLElementEventMap['${node.name}'])=>void`));
    return directive;
}
/** Convert for Class Directive */
function convertClassDirective(node, parent, ctx) {
    const directive = Object.assign({ type: "SvelteDirective", kind: "Class", name: null, modifiers: node.modifiers, expression: null, parent }, ctx.getConvertLocation(node));
    processDirective(node, directive, ctx, buildProcessExpressionForExpression(directive, ctx, null));
    return directive;
}
/** Convert for Transition Directive */
function convertTransitionDirective(node, parent, ctx) {
    const directive = Object.assign({ type: "SvelteDirective", kind: "Transition", intro: node.intro, outro: node.outro, name: null, modifiers: node.modifiers, expression: null, parent }, ctx.getConvertLocation(node));
    processDirective(node, directive, ctx, buildProcessExpressionForExpression(directive, ctx, null), (name) => ctx.scriptLet.addExpression(name, directive));
    return directive;
}
/** Convert for Animation Directive */
function convertAnimationDirective(node, parent, ctx) {
    const directive = Object.assign({ type: "SvelteDirective", kind: "Animation", name: null, modifiers: node.modifiers, expression: null, parent }, ctx.getConvertLocation(node));
    processDirective(node, directive, ctx, buildProcessExpressionForExpression(directive, ctx, null), (name) => ctx.scriptLet.addExpression(name, directive));
    return directive;
}
/** Convert for Action Directive */
function convertActionDirective(node, parent, ctx) {
    const directive = Object.assign({ type: "SvelteDirective", kind: "Action", name: null, modifiers: node.modifiers, expression: null, parent }, ctx.getConvertLocation(node));
    processDirective(node, directive, ctx, buildProcessExpressionForExpression(directive, ctx, null), (name) => ctx.scriptLet.addExpression(name, directive));
    return directive;
}
/** Convert for Let Directive */
function convertLetDirective(node, parent, ctx) {
    const directive = Object.assign({ type: "SvelteDirective", kind: "Let", name: null, modifiers: node.modifiers, expression: null, parent }, ctx.getConvertLocation(node));
    processDirective(node, directive, ctx, (pattern) => {
        return ctx.letDirCollections
            .getCollection()
            .addPattern(pattern, directive, "any");
    }, node.expression
        ? undefined
        : (name) => {
            // shorthand
            return ctx.letDirCollections
                .getCollection()
                .addPattern(name, directive, "any", (es) => {
                directive.expression = es;
            });
        });
    return directive;
}
/** Common process for directive */
function processDirective(node, directive, ctx, processExpression, processName) {
    const colonIndex = ctx.code.indexOf(":", directive.range[0]);
    ctx.addToken("HTMLIdentifier", {
        start: directive.range[0],
        end: colonIndex,
    });
    const nameIndex = ctx.code.indexOf(node.name, colonIndex + 1);
    const nameRange = {
        start: nameIndex,
        end: nameIndex + node.name.length,
    };
    // modifiers
    if (ctx.code[nameRange.end] === "|") {
        let nextStart = nameRange.end + 1;
        let nextEnd = common_1.indexOf(ctx.code, (c) => c === "=" || c === ">" || c === "|" || !c.trim(), nextStart);
        ctx.addToken("HTMLIdentifier", { start: nextStart, end: nextEnd });
        while (ctx.code[nextEnd] === "|") {
            nextStart = nextEnd + 1;
            nextEnd = common_1.indexOf(ctx.code, (c) => c === "=" || c === ">" || c === "|" || !c.trim(), nextStart);
            ctx.addToken("HTMLIdentifier", { start: nextStart, end: nextEnd });
        }
    }
    let isShorthand = false;
    if (node.expression) {
        isShorthand =
            node.expression.type === "Identifier" &&
                node.expression.name === node.name &&
                common_1.getWithLoc(node.expression).start === nameRange.start &&
                common_1.getWithLoc(node.expression).end === nameRange.end;
        processExpression(node.expression).push((es) => {
            directive.expression = es;
            if (isShorthand) {
                directive.name = directive.expression;
            }
        });
    }
    // put name
    if (!isShorthand) {
        directive.name = Object.assign({ type: "Identifier", name: node.name, 
            // @ts-expect-error -- ignore
            parent: directive }, ctx.getConvertLocation(nameRange));
        if (processName) {
            processName(directive.name).push((es) => {
                directive.name = es;
            });
        }
        else {
            ctx.addToken("HTMLIdentifier", nameRange);
        }
    }
}
/** Build processExpression for Expression */
function buildProcessExpressionForExpression(directive, ctx, typing) {
    return (expression) => {
        return ctx.scriptLet.addExpression(expression, directive, typing);
    };
}
